<?php // $Id: pay_method_gateway_payflowpro.inc,v 1.1 2010/11/01 20:51:15 vauxia Exp $

/**
 * @file
 * TODO:
 * -  Not documenting/using any telecheck fields because it seems to require
 * more data (SSN, Driver's License, etc.) Than is practical for a Drupal site.
 * - Ignoring all of the custom air travel fields, because WTF?
 * - Omitting support for purchasing cards/PO's.
 */

class pay_method_gateway_payflowpro extends pay_method_gateway {

  // Settings specific to Payflow Pro.
  var $payflowpro_partner;
  var $payflowpro_vendor;
  var $payflowpro_user;
  var $payflowpro_pwd;

  var $gateway_supports_ach = FALSE;
  var $gateway_supports_cc = TRUE;
  var $gateway_supports_recurring = TRUE;

  function gateway_url() {
    if ($this->gateway_testmode) {
      return 'https://pilot-payflowpro.paypal.com';
    }
    return'https://payflowpro.paypal.com';
  }

  function gateway_request() {

    // Set the transaction type based on requested activity.
    if (!$txntype = $this->payflowpro_trxtype($this->activity->activity)) {
      drupal_set_message("Payment activity '$this->activity->activity' is unsupported. Not processing transaction", 'error');
      return FALSE;
    }

    $request = array();
    foreach ($this->gateway_request_values($txntype) as $key => $val) {
      $val = check_plain(str_replace('"', '', trim($val)));
      // Include a string length in case the value contains special characters.
      if($len = strlen($val)) {
        $request[] = $key .'['. $len .']='. $val;
      }
    }
    return join('&', $request) .'"';
  }

  function gateway_headers() {
    return array(
      'X-VPS-Request-Id' => $this->activity->paid,
      'X-VPS-Timeout' => 45,
      'X-VPS-VIT-Integration-Product' => t('Pay module for Drupal'),
      'Connection' => 'close',
      'Content-Type' => 'text/namevalue',
    );
  }

  /*
    RESULT: 0 or error code: <0 communication errors, 0< transaction errors.
    CVV2MATCH: Y, N, X or no response.
    RESPMSG: Response/error message.
    AUTHCODE: Returned for Sale, Authorization, and Voice Authorizations.
    AVSADDR: Y, N, X, or no response.
    AVSZIP: Y, N, X, or no response.
    IAVS: Y, N, X, or no response. Indicates whether address is international
  */
  function gateway_response($result) {
    $data = array();
    parse_str($result, $data);

    // Save the transaction ID for tracking and/or future operations.
    $this->activity->identifier = $data['PNREF'];

    // Some kind of error occurred.
    if ($data['RESULT'] != 0) {
      $this->activity->data = $data;
      watchdog('payment', "Error processing payment: Payflow Pro gateway returned '@err'", array('@err' => $data['RESPMSG']));
      drupal_set_message("Error processing payment: Payflow Pro gateway returned '". $data['RESPMSG']. "'");
      return FALSE;
    }

    // Store the response in the 'data' array for tracking and forensics.
    unset($data['RESULT'], $data['RESPMSG'], $data['PNREF']);
    $this->activity->data = $data;

    return TRUE;
  }

  function settings_form(&$form, &$form_state) {
    parent::settings_form($form, $form_state);

    $group = $this->handler();
    $form[$group]['#type'] = 'fieldset';
    $form[$group]['#collapsible'] = FALSE;
    $form[$group]['#title'] = t('Payflow Pro settings');

    // For consistency, these field labels are from https://manager.paypal.com/
    $form[$group]['payflowpro_partner'] = array(
      '#type' => 'textfield',
      '#title' => t('Partner'),
      '#description' => t('The ID provided to you by the authorized PayPal Reseller who registered you for Payflow Pro. If you purchased your account directly from PayPal, use PayPal.'),
      '#default_value' => $this->payflowpro_partner,
      '#required' => TRUE,
    );
    $form[$group]['payflowpro_vendor'] = array(
      '#type' => 'textfield',
      '#title' => t('Merchant login'),
      '#description' => t('Your merchant login ID that you created when you registered for the account.'),
      '#default_value' => $this->payflowpro_vendor,
      '#required' => TRUE,
    );
    $form[$group]['payflowpro_user'] = array(
      '#type' => 'textfield',
      '#title' => t('User'),
      '#description' => t('The ID provided to you by the authorized PayPal Reseller who registered you for Payflow Pro. If you purchased your account directly from PayPal, use PayPal.'),
      '#default_value' => $this->payflowpro_user,
    );
    $form[$group]['payflowpro_pwd'] = array(
      '#type' => 'textfield',
      '#title' => t('Password'),
      '#description' => t('The password that you defined while registering for the account.'),
      '#default_value' => $this->payflowpro_pwd,
      '#required' => TRUE,
    );
  }

  function payflowpro_trxtype($activity) {
    // S:Sale C:Credit A:Auth D:Delayed V:Void F:Voice I:Inquiry N:Duplicate
    $trxtypes = array(
      'complete'  => 'S',
      'authorize' => 'A',
      'capture'   => 'D',
      'void'      => 'V',
      'credit'    => 'C',
    );
    return $trxtypes[$activity];
  }

  // TODO
  function gateway_request_values($txntype) {
    $data = array(
      // User parameters: Required.
      'USER'           => $this->payflowpro_user ? $this->payflowpro_user : $this->payflowpro_vendor,
      'VENDOR'         => $this->payflowpro_vendor,
      'PARTNER'        => $this->payflowpro_partner,
      'PWD'            => $this->payflowpro_pwd,

      // Sale transaction: Required.
      'TRXTYPE'        => $txntype,
      'TENDER'         => 'C', // A:ACH C:Credit D:Debit K:Telecheck P:Paypal
      'ACCT'           => $this->cc_number, // Required for TRXTYPE:S
      'AMT'            => $this->total(), // Amount, float.
      'EXPDATE'        => $this->cc_expiration(), // Expiration (MMYY)

      // Additional information: Optional, but probably reduces discount rate.
      'CVV2'           => $this->cc_ccv2, // (3-4 digits)
      'FIRSTNAME'      => $this->first_name, // (30 chars)
      'LASTNAME'       => $this->last_name, // (30 chars)
      'STREET'         => $this->billto['street1'], // (30 chars)
      'ZIP'            => $this->billto['zip'], // (5-9 chars)

      // Required for Capture, Void, Inquiry, after Sale or Auth transaction.
      'ORIGID'         => $this->identifier, // (12 chars)

      // Inquiry transaction using the CUSTREF field, also includes these.
      'STARTTIME'      => '', // (14 chars: yyyymmddhhmmss)
      'ENDTIME'        => '', // (14 chars) Must be within 30 days of start.

      // Additional information: Optional.
      'INVNUM'         => '', // (9 chars) Invoice number.
      'ORDERDATE'      => '', // (7 chars)
      'ORDERDATETIME'  => '', // (19 chars)
      'COMMENT1'       => truncate_utf8($this->note, 128), // (128 chars)
      'COMMENT2'       => '', // (128 chars)
      'EMAIL'          => $this->mail, // (60 chars)
      'PHONENUM'       => '', // (20 chars)
      'PHONETYPE'      => '', // (2 chars) ANI indicating call type.
      'CUSTHOSTNAME'   => '', // (60 chars) remote hostname.
      'CUSTBROWSER'    => '', // (60 chars) user agent.
      'CUSTIP'         => '', // (15 chars)
      'SHIPTOFIRSTNAME' => '', // (30 chars)
      'SHIPTOLASTNAME' => '', // (30 chars)
      'SHIPTOSTREET'   => '', // (30 chars)
      'SHIPTOCOUNTRY'  => '', // (3 chars) Numeric code.
      'SHIPTOZIP'      => '', // (5-9 chars)
      'SHIPTOPHONENUM' => '', // (10 chars)
      'SHIPMETHOD'     => '', // (2 chars) 01,02,03,04,05
      'SKU'            => '', // (15 chars)
      'RECURRING'      => '', // (1 char) Y, N

      // Merchant information.
      'MERCHANTNAME'    => '', // 
      'MERCHANTSTREET'  => '', // (30 chars)
      'MERCHANTCITY'    => '', //
      'MERCHANTSTATE'   => '', //
      'MERCHANTCOUNTRYCODE'   => '', // (3 chars, numeric)
      'MERCHANTZIP'     => '', // (5-9 chars)

      // "Additional amounts", such as tip, convenience charge, etc.
      'ADDLAMT1'        => '', // (12 chars), float
      'ADDLAMTTYPE1'    => '', // (3 chars)
      'ADDLAMT2'        => '', // (12 chars), float
      'ADDLAMTTYPE2'    => '', // (3 chars)
      'ADDLAMT3'        => '', // (12 chars), float
      'ADDLAMTTYPE3'    => '', // (3 chars)
      'ADDLAMT4'        => '', // (12 chars), float
      'ADDLAMTTYPE4'    => '', // (3 chars)
      'ADDLAMT5'        => '', // (12 chars), float
      'ADDLAMTTYPE5'    => '', // (3 chars)

      // Description, may appear on customer statements depending on procesor.
      'DESC'            => '', // (19 chars)

      // Reporting parameters. Appear in the Manager section but aren't used.
      'BILLTOCOUNTRY'   => '', // (3 chars) Numeric code.
      'CUSTCODE'        => '', // (4 chars)
      'DUTYAMT'         => '', // (10 chars)
      'TAXAMT'          => '', // (10 chars)
      'FREIGHTAMT'      => '', // (10 chars)
      'NAME'            => '', // (15 chars)
      'PONUM'           => '', // (15 chars)
      'SHIPTOSTATE'     => '', // (10 chars) 2-digit (US) or full name.
      'STATE'           => $this->billto['state'], // (2 chars) 
    );
    return $data;
  }
}
